@@ -23,12 +23,13 @@
 // #define DEBUG 1
 
 #include "avcodec.h"
+#include "copy_block.h"
 #include "bytestream.h"
 #include "internal.h"
 #include "libavutil/bswap.h"
 #include "libavutil/imgutils.h"
-#include "libavcodec/dsputil.h"
 #include "sanm_data.h"
+#include "libavutil/avassert.h"
 
 #define NGLYPHS 256
 
@@ -45,7 +46,7 @@ typedef struct {
     int aligned_width, aligned_height;
     int prev_seq;
 
-    AVFrame frame, *output;
+    AVFrame *frame;
     uint16_t *frm0, *frm1, *frm2;
     uint8_t *stored_frame;
     uint32_t frm0_size, frm1_size, frm2_size;
@@ -230,6 +231,9 @@ static void destroy_buffers(SANMVideoContext *ctx)
     av_freep(&ctx->frm2);
     av_freep(&ctx->stored_frame);
     av_freep(&ctx->rle_buf);
+    ctx->frm0_size =
+    ctx->frm1_size =
+    ctx->frm2_size = 0;
 }
 
 static av_cold int init_buffers(SANMVideoContext *ctx)
@@ -270,8 +274,6 @@ static av_cold int decode_init(AVCodecContext *avctx)
         av_log(avctx, AV_LOG_ERROR, "error allocating buffers\n");
         return AVERROR(ENOMEM);
     }
-    ctx->output          = &ctx->frame;
-    ctx->output->data[0] = 0;
 
     make_glyphs(ctx->p4x4glyphs[0], glyph4_x, glyph4_y, 4);
     make_glyphs(ctx->p8x8glyphs[0], glyph8_x, glyph8_y, 8);
@@ -298,11 +300,6 @@ static av_cold int decode_end(AVCodecContext *avctx)
 
     destroy_buffers(ctx);
 
-    if (ctx->frame.data[0]) {
-        avctx->release_buffer(avctx, &ctx->frame);
-        ctx->frame.data[0] = 0;
-    }
-
     return 0;
 }
 
@@ -618,6 +615,16 @@ static int process_block(SANMVideoContext *ctx, uint8_t *dst, uint8_t *prev1,
     } else {
         int mx = motion_vectors[code][0];
         int my = motion_vectors[code][1];
+        int index = prev2 - (const uint8_t*)ctx->frm2;
+
+        av_assert2(index >= 0 && index < (ctx->buf_size>>1));
+
+        if (index < - mx - my*stride ||
+            (ctx->buf_size>>1) - index < mx + size + (my + size - 1)*stride) {
+            av_log(ctx->avctx, AV_LOG_ERROR, "MV is invalid \n");
+            return AVERROR_INVALIDDATA;
+        }
+
         for (k = 0; k < size; k++)
             memcpy(dst + k * stride, prev2 + mx + (my + k) * stride, size);
     }
@@ -662,8 +669,7 @@ static int old_codec47(SANMVideoContext *ctx, int top,
         if (bytestream2_get_bytes_left(&ctx->gb) < width * height)
             return AVERROR_INVALIDDATA;
         for (j = 0; j < height; j++) {
-            for (i = 0; i < width; i++)
-                bytestream2_get_bufferu(&ctx->gb, dst, width);
+            bytestream2_get_bufferu(&ctx->gb, dst, width);
             dst += stride;
         }
         break;
@@ -726,13 +732,19 @@ static int process_frame_obj(SANMVideoContext *ctx)
     w     = bytestream2_get_le16u(&ctx->gb);
     h     = bytestream2_get_le16u(&ctx->gb);
 
+    if (!w || !h) {
+        av_log(ctx->avctx, AV_LOG_ERROR, "dimensions are invalid\n");
+        return AVERROR_INVALIDDATA;
+    }
+
     if (ctx->width < left + w || ctx->height < top + h) {
         if (av_image_check_size(FFMAX(left + w, ctx->width),
                                 FFMAX(top  + h, ctx->height), 0, ctx->avctx) < 0)
             return AVERROR_INVALIDDATA;
         avcodec_set_dimensions(ctx->avctx, FFMAX(left + w, ctx->width),
                                            FFMAX(top  + h, ctx->height));
-        init_sizes(ctx, left + w, top + h);
+        init_sizes(ctx, FFMAX(left + w, ctx->width),
+                        FFMAX(top  + h, ctx->height));
         if (init_buffers(ctx)) {
             av_log(ctx->avctx, AV_LOG_ERROR, "error resizing buffers\n");
             return AVERROR(ENOMEM);
@@ -753,7 +765,7 @@ static int process_frame_obj(SANMVideoContext *ctx)
         return old_codec47(ctx, top, left, w, h);
         break;
     default:
-        av_log_ask_for_sample(ctx->avctx, "unknown subcodec %d\n", codec);
+        avpriv_request_sample(ctx->avctx, "unknown subcodec %d", codec);
         return AVERROR_PATCHWELCOME;
     }
 }
@@ -777,7 +789,7 @@ static int decode_0(SANMVideoContext *ctx)
 
 static int decode_nop(SANMVideoContext *ctx)
 {
-    av_log_ask_for_sample(ctx->avctx, "unknown/unsupported compression type\n");
+    avpriv_request_sample(ctx->avctx, "unknown/unsupported compression type");
     return AVERROR_PATCHWELCOME;
 }
 
@@ -1032,8 +1044,10 @@ static int decode_5(SANMVideoContext *ctx)
 #if HAVE_BIGENDIAN
     npixels = ctx->npixels;
     frm = ctx->frm0;
-    while (npixels--)
-        *frm++ = av_bswap16(*frm);
+    while (npixels--) {
+        *frm = av_bswap16(*frm);
+        frm++;
+    }
 #endif
 
     return 0;
@@ -1137,13 +1151,11 @@ static int copy_output(SANMVideoContext *ctx, SANMFrameHeader *hdr)
     int ret, dstpitch, height = ctx->height;
     int srcpitch = ctx->pitch * (hdr ? sizeof(ctx->frm0[0]) : 1);
 
-    if ((ret = ff_get_buffer(ctx->avctx, ctx->output)) < 0) {
-        av_log(ctx->avctx, AV_LOG_ERROR, "get_buffer() failed\n");
+    if ((ret = ff_get_buffer(ctx->avctx, ctx->frame, 0)) < 0)
         return ret;
-    }
 
-    dst      = ctx->output->data[0];
-    dstpitch = ctx->output->linesize[0];
+    dst      = ctx->frame->data[0];
+    dstpitch = ctx->frame->linesize[0];
 
     while (height--) {
         memcpy(dst, src, srcpitch);
@@ -1160,9 +1172,8 @@ static int decode_frame(AVCodecContext *avctx, void *data,
     SANMVideoContext *ctx = avctx->priv_data;
     int i, ret;
 
+    ctx->frame = data;
     bytestream2_init(&ctx->gb, pkt->data, pkt->size);
-    if (ctx->output->data[0])
-        avctx->release_buffer(avctx, ctx->output);
 
     if (!ctx->version) {
         int to_store = 0;
@@ -1244,7 +1255,7 @@ static int decode_frame(AVCodecContext *avctx, void *data,
             memcpy(ctx->stored_frame, ctx->frm0, ctx->buf_size);
         if ((ret = copy_output(ctx, NULL)))
             return ret;
-        memcpy(ctx->output->data[1], ctx->pal, 1024);
+        memcpy(ctx->frame->data[1], ctx->pal, 1024);
     } else {
         SANMFrameHeader header;
 
@@ -1252,12 +1263,12 @@ static int decode_frame(AVCodecContext *avctx, void *data,
             return ret;
 
         ctx->rotate_code = header.rotate_code;
-        if ((ctx->output->key_frame = !header.seq_num)) {
-            ctx->output->pict_type = AV_PICTURE_TYPE_I;
+        if ((ctx->frame->key_frame = !header.seq_num)) {
+            ctx->frame->pict_type = AV_PICTURE_TYPE_I;
             fill_frame(ctx->frm1, ctx->npixels, header.bg_color);
             fill_frame(ctx->frm2, ctx->npixels, header.bg_color);
         } else {
-            ctx->output->pict_type = AV_PICTURE_TYPE_P;
+            ctx->frame->pict_type = AV_PICTURE_TYPE_P;
         }
 
         if (header.codec < FF_ARRAY_ELEMS(v1_decoders)) {
@@ -1267,7 +1278,7 @@ static int decode_frame(AVCodecContext *avctx, void *data,
                 return ret;
             }
         } else {
-            av_log_ask_for_sample(avctx, "subcodec %d is not implemented\n",
+            avpriv_request_sample(avctx, "subcodec %d",
                    header.codec);
             return AVERROR_PATCHWELCOME;
         }
@@ -1279,13 +1290,13 @@ static int decode_frame(AVCodecContext *avctx, void *data,
         rotate_bufs(ctx, ctx->rotate_code);
 
     *got_frame_ptr  = 1;
-    *(AVFrame*)data = *ctx->output;
 
     return pkt->size;
 }
 
 AVCodec ff_sanm_decoder = {
     .name           = "sanm",
+    .long_name      = NULL_IF_CONFIG_SMALL("LucasArts SMUSH video"),
     .type           = AVMEDIA_TYPE_VIDEO,
     .id             = AV_CODEC_ID_SANM,
     .priv_data_size = sizeof(SANMVideoContext),
@@ -1293,5 +1304,4 @@ AVCodec ff_sanm_decoder = {
     .close          = decode_end,
     .decode         = decode_frame,
     .capabilities   = CODEC_CAP_DR1,
-    .long_name      = NULL_IF_CONFIG_SMALL("LucasArts SMUSH video"),
 };
