@@ -125,9 +125,7 @@ static av_cold int rl2_read_header(AVFormatContext *s)
     if(signature == RLV3_TAG && back_size > 0)
         st->codec->extradata_size += back_size;
 
-    st->codec->extradata = av_mallocz(st->codec->extradata_size +
-                                          FF_INPUT_BUFFER_PADDING_SIZE);
-    if(!st->codec->extradata)
+    if(ff_alloc_extradata(st->codec, st->codec->extradata_size))
         return AVERROR(ENOMEM);
 
     if(avio_read(pb,st->codec->extradata,st->codec->extradata_size) !=
@@ -136,8 +134,10 @@ static av_cold int rl2_read_header(AVFormatContext *s)
 
     /** setup audio stream if present */
     if(sound_rate){
-        if(channels <= 0)
+        if (!channels || channels > 42) {
+            av_log(s, AV_LOG_ERROR, "Invalid number of channels: %d\n", channels);
             return AVERROR_INVALIDDATA;
+        }
 
         pts_num = def_sound_size;
         pts_den = rate;
